/*
 * Expressions evaluation program
 * Thomas Nemeth, 2003
 */


#ifndef LISTS_H
#define LISTS_H

#define MAXSTRLEN 2048

/* General replies */
#define OK  1
#define ERR 0


/* Parse errors */
#define CHECKERROR      1
#define PARSEERROR     10 /* + character position */

/* Evaluation errors */
#define UNKNOWNOPER     2
#define DIVISIONBYZERO  3
#define OUTOFRANGE      4
#define UNKNOWNFUNC     5

/* Loading errors */
#define UNKNOWNFILE     6
#define UNKNOWNENTRY    7
#define READERROR       8
#define VARLISTERR    100 /* + var_num */
#define FCTLISTERR   1000 /* x fct_num + other errors */
#define CFGFILEERR  10000 /* x line_num + other errors */


typedef enum {NOTHING, VALUE, VARIABLE, OPERATOR, FUNCTION, USERF} Type;
typedef enum {S_NONE, S_VALU, S_IDEN, S_OPER, S_OPAR, S_CPAR} State;

typedef struct ttok_ {
    char  *token;
    double value;
    Type   type;
    int    priority;
    int    id;
    struct ttok_ *next;
} TTok;


typedef struct trng_ {
    char   *name;
    double  from;
    double  to;
    double  step;
    int     eval;
    int     actv;
    int     main;
    struct trng_ *next;
} TRng;


typedef struct tfct_ {
    TTok *head;
    TTok *queue;
    char *name;
    char *expr;
    char *var;
    int   eval;
    struct tfct_ *next;
} TFct;

typedef TFct TVar;


typedef struct tenv_ {
    TFct *fct_list_head;
    TFct *fct_list_queue;
    TVar *var_list;
    TRng *rng_list_head;
    TRng *rng_list_queue;
    int   precision;
} TEnv;


TFct* init_list(char *expr);
TEnv* init_env();
void add_tok(TFct *list, TTok *tok);
void add_elt(TFct *list, const char *token, Type type, int priority, int id);
void add_res(TFct *list, double var, double val, TEnv *env);
void add_list(TFct *list, TFct *add);
int exist_elt(TVar *list, const char *name);
void push_copy(TFct *list, TTok *t);
void push_res(TFct *list, double value);
TTok* pop_elt(TFct *list);
double pop_value(TFct *list);
void pop_list(TFct *dest, TFct *src);
void add_var(TEnv *env, const char *name);
void set_listvar_value(TFct *list, const char *name, double value);
void set_var_value(TEnv *env, const char *name, double value);
double find_var_value(TEnv *env, const char *name);
void add_fct(TEnv *env, TFct *fct, const char *name, const char *var);
void add_rng(TEnv *env, const char *name, double from, double to, double step, double ismain);
void free_elt(TTok **elt);
void free_list(TFct **list);
void free_env(TEnv **env);

#endif
